#!/usr/bin/env python
import gzip
import shutil
import tarfile
import os.path
from argparse import ArgumentParser

from ebi_eva_common_pyutils.logger import logging_config
from retry import retry

logger = logging_config.get_logger(__name__)


def make_tarfile(output_filename, source_dir):
    logger.info(f'Create Final Tar file {output_filename}.')
    with tarfile.open(output_filename, "w") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))
    file_stats = os.stat(output_filename)
    logger.info(f'{output_filename} completed. File Size in Bytes is {file_stats.st_size}')


def is_compressed(file_path):
    compressed_ext = ['.gz', '.zip', '.bz', '.tbi', '.csi']
    for ext in compressed_ext:
        if file_path.lower().endswith(ext):
            return True
    return False


@retry(tries=5, delay=3, backoff=2, logger=logger)
def retriable_compress(src_file_path, dest_file_path):
    MEG = 2 ** 20
    with open(src_file_path, 'rb') as f_in:
        with gzip.open(dest_file_path, 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out, length=16 * MEG)


def matches(name, patterns):
    return any((pattern for pattern in patterns if pattern in name))


@retry(tries=5, delay=3, backoff=2, logger=logger)
def retriable_remove(path):
    shutil.rmtree(path)


@retry(tries=5, delay=3, backoff=2, logger=logger)
def retryable_copy(src_file_path, dest_file_path, **kwargs):
    if os.path.exists(dest_file_path):
        os.remove(dest_file_path)
    shutil.copyfile(src_file_path, dest_file_path, **kwargs)


def archive_directory(source_dir, scratch_dir, destination_dir, filter_patterns=None):
    """
    Archive a directory by copying the data it contains to a scratch directory compressing all files that are not
    already compressed. Then it create a tar file to the destination_dir under the name of the original directory.
    """
    source_dir_name = os.path.basename(source_dir)
    logger.info(f'Archive {source_dir_name} from {source_dir}')
    parent_source_dir = os.path.dirname(source_dir)
    for base, dirs, files in os.walk(source_dir, topdown=True, followlinks=False):
        # Filter the downstream directory to
        filtered_dir = []
        for d in dirs:
            if matches(d, filter_patterns):
                logger.info(f'Ignore directory {d} because of filters: {filter_patterns}')
            else:
                filtered_dir.append(d)
        # modify dirs in place
        dirs[:] = filtered_dir
        src_basename = os.path.relpath(base, parent_source_dir)
        scratch_dest_dir = os.path.join(scratch_dir, src_basename)
        os.makedirs(scratch_dest_dir, exist_ok=True)
        for fname in files:
            src_file_path = os.path.join(base, fname)
            dest_file_path = os.path.join(scratch_dest_dir, fname)
            if matches(fname, filter_patterns):
                logger.info(f'Ignore file {src_file_path} because of filters: {filter_patterns}')
                continue
            if os.path.islink(src_file_path) or is_compressed(src_file_path):
                logger.info(f'Copy {src_file_path}')
                retryable_copy(src_file_path, dest_file_path, follow_symlinks=False)
            else:
                logger.info(f'Compress {src_file_path}')
                retriable_compress(src_file_path, dest_file_path + '.gz')
    final_tar_file = os.path.join(destination_dir, source_dir_name + '.tar')
    scratch_dir_archived = os.path.join(scratch_dir, source_dir_name)
    make_tarfile(final_tar_file, scratch_dir_archived)
    logger.info(f'Delete scratch folder {scratch_dir_archived}.')
    retriable_remove(scratch_dir_archived)
    logger.info(f'Scratch folder {scratch_dir_archived} deleted.')


def main():
    parser = ArgumentParser(description='Archive a directory by copying the data it contains to a scratch directory '
                                        'compressing all files that are not already compressed. Then it create a tar '
                                        'file to the destination_dir under the name of the original directory.')
    parser.add_argument('--source_dir', required=True, type=str,
                        help='base directory you want to archive. All sub directories will be included.')
    parser.add_argument('--destination_dir', required=True, type=str,
                        help='Directory where the archive should be placed at the end of the process.')
    parser.add_argument('--scratch_dir', required=True, type=str,
                        help='Directory where the archive will be constructed.')
    parser.add_argument('--filter_patterns', type=str, nargs='*', default=[],
                        help='keyword found in file and directory names that not be included to the archive.')
    args = parser.parse_args()

    logging_config.add_stdout_handler()
    archive_directory(args.source_dir, args.scratch_dir,  args.destination_dir, args.filter_patterns)


if __name__ == '__main__':
    main()



import os

import yaml


class Configuration:
    """
    Configuration class that allow to load a yaml file either at construction or later in the execution.
    It can be used like a dict but should be used as readonly.
    """
    config_file = None
    content = {}

    def __init__(self, *search_path):
        if search_path:
            self.load_config_file(*search_path)

    @staticmethod
    def _find_config_file(search_path):
        for p in search_path:
            if p and os.path.isfile(p):
                return p

    def load_config_file(self, *search_path):
        self.config_file = self._find_config_file(search_path)
        if self.config_file:
            with open(self.config_file, 'r') as f:
                self.content = yaml.safe_load(f)
        else:
            raise FileNotFoundError('Could not find any config file in specified search path')

    def get(self, item, ret_default=None):
        """
        Dict-style item retrieval with default
        :param item: The key to search for
        :param ret_default: What to return if the key is not present
        """
        try:
            return self[item]
        except KeyError:
            return ret_default

    def query(self, *parts, ret_default=None):
        """
        Drill down into a config, e.g. cfg.query('logging', 'handlers', 'a_handler', 'level')
        :param ret_default:
        :return: The relevant item if it exists in the config, else ret_default.
        """
        top_level = self.content
        item = None

        for p in parts:
            item = top_level.get(p)
            if item:
                top_level = item
            else:
                return ret_default
        return item

    def report(self):
        return yaml.safe_dump(self.content, default_flow_style=False)

    def __getitem__(self, item):
        """Allow dict-style access, e.g. config['this'] or config['this']['that']."""
        return self.content[item]

    def __contains__(self, item):
        """Allow search in the first layer of the config with 'in' operator."""
        return self.content.__contains__(item)


cfg = Configuration()
"""
Provides a singleton that can be used as a central place for configuration.
"""




class WritableConfig(Configuration):
    """Configuration object that allows writes to the config file"""

    def __init__(self, *search_path, version=None):
        super().__init__(*search_path)
        self.version = version

    def load_config_file(self, *search_path):
        try:
            super().load_config_file(*search_path)
        except FileNotFoundError:
            # expected if it's the first time we are creating the config file
            # In that case the first search path is set to be the config files
            self.config_file = search_path[0]
            pass

    def backup(self):
        """
        Rename the config file by adding a '.1' at the end. If the '.1' file exists it move it to a '.2' and so on.
        """
        if os.path.isfile(self.config_file):
            file_name = self.config_file
            suffix = 1
            backup_name = f'{file_name}.{suffix}'
            while os.path.exists(backup_name):
                suffix += 1
                backup_name = f'{file_name}.{suffix}'

            for i in range(suffix, 1, -1):
                os.rename(f'{file_name}.{i - 1}', f'{file_name}.{i}')
            os.rename(file_name, file_name + '.1')

    def write(self):
        if self.config_file and self.content and os.path.isdir(os.path.dirname(self.config_file)):
            with open(self.config_file, 'w') as open_config:
                yaml.safe_dump(self.content, open_config)

    def set(self, *path, value):
        self._set_version()
        top_level = self.content
        for p in path[:-1]:
            if p not in top_level:
                top_level[p] = {}
            top_level = top_level[p]
        top_level[path[-1]] = value

    def pop(self, *path, default=None):
        """Recursive dictionary pop with default"""
        top_level = self.content
        for p in path[:-1]:
            if p not in top_level:
                return default
            top_level = top_level[p]
        return top_level.pop(path[-1], default)

    def is_empty(self):
        return not self.content

    def clear(self):
        self.content = {}

    def _set_version(self):
        # If we're starting to fill in an empty config, set the version if available
        if self.is_empty() and self.version:
            self.content['version'] = self.version

    def __contains__(self, item):
        return item in self.content

    def __setitem__(self, item, value):
        """Allow dict-style write access, e.g. config['this']='that'."""
        self._set_version()
        self.content[item] = value


# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import datetime
import re
from urllib.parse import urlsplit

import psycopg2

from ebi_eva_common_pyutils.assembly_utils import is_patch_assembly
from ebi_eva_internal_pyutils.config_utils import get_metadata_creds_for_profile
from ebi_eva_common_pyutils.ena_utils import get_scientific_name_and_common_name
from ebi_eva_common_pyutils.logger import logging_config
from ebi_eva_common_pyutils.ncbi_utils import get_ncbi_assembly_name_from_term
from ebi_eva_internal_pyutils.pg_utils import get_result_cursor, get_all_results_for_query, execute_query
from ebi_eva_common_pyutils.taxonomy.taxonomy import get_scientific_name_from_ensembl

logger = logging_config.get_logger(__name__)
SUPPORTED_ASSEMBLY_TRACKER_TABLE = "evapro.supported_assembly_tracker"


def get_metadata_connection_handle(profile, settings_xml_file):
    pg_url, pg_user, pg_pass = get_metadata_creds_for_profile(profile, settings_xml_file)
    return psycopg2.connect(urlsplit(pg_url).path, user=pg_user, password=pg_pass)


def get_db_conn_for_species(species_db_info):
    db_name = "dbsnp_{0}".format(species_db_info["dbsnp_build"])
    pg_conn = psycopg2.connect("dbname='{0}' user='{1}' host='{2}'  port={3}".
                               format(db_name, "dbsnp", species_db_info["pg_host"], species_db_info["pg_port"]))
    return pg_conn


def get_species_info(metadata_connection_handle, dbsnp_species_name="all"):
    get_species_info_query = "SELECT DISTINCT database_name, scientific_name, dbsnp_build, pg_host, pg_port " \
                             "FROM dbsnp_ensembl_species.import_progress a " \
                             "JOIN dbsnp_ensembl_species.dbsnp_build_instance b " \
                             "ON b.dbsnp_build = a.ebi_pg_dbsnp_build "
    if dbsnp_species_name != "all":
        get_species_info_query += "where database_name = '{0}' ".format(dbsnp_species_name)
    get_species_info_query += "order by database_name"

    pg_cursor = get_result_cursor(metadata_connection_handle, get_species_info_query)
    species_set = [{"database_name": result[0], "scientific_name": result[1], "dbsnp_build":result[2],
                    "pg_host":result[3], "pg_port":result[4]}
                   for result in pg_cursor.fetchall()]
    pg_cursor.close()
    return species_set


# Get connection information for each Postgres instance of the dbSNP mirror
def get_dbsnp_mirror_db_info(pg_metadata_dbname, pg_metadata_user, pg_metadata_host):
    with psycopg2.connect("dbname='{0}' user='{1}' host='{2}'".format(pg_metadata_dbname, pg_metadata_user,
                                                                      pg_metadata_host)) as pg_conn:
        dbsnp_mirror_db_info_query = "SELECT * FROM dbsnp_ensembl_species.dbsnp_build_instance"
        dbsnp_mirror_db_info = [{"dbsnp_build": result[0], "pg_host": result[1], "pg_port": result[2]}
                                for result in get_all_results_for_query(pg_conn, dbsnp_mirror_db_info_query)]
    return dbsnp_mirror_db_info


def get_taxonomy_code_from_metadata(metadata_connection_handle, taxonomy):
    """
    Retrieve an existing taxonomy code registered in the metadata database.
    """
    query = f"SELECT DISTINCT t.taxonomy_code FROM taxonomy t WHERE t.taxonomy_id = {taxonomy}"
    rows = get_all_results_for_query(metadata_connection_handle, query)
    if len(rows) == 0:
        return None
    elif len(rows) > 1:
        options = ', '.join(rows)
        raise ValueError(f'More than one possible code for taxonomy {taxonomy} found: {options}')
    return rows[0][0]


def get_assembly_code_from_metadata(metadata_connection_handle, assembly):
    """
    Retrieve an existing assembly code registered in the metadata database.
    """
    query = f"SELECT DISTINCT assembly_code FROM assembly WHERE assembly_accession='{assembly}'"
    rows = get_all_results_for_query(metadata_connection_handle, query)
    if len(rows) == 0:
        return None
    elif len(rows) > 1:
        options = ', '.join([row for row, in rows])
        raise ValueError(f'More than one possible code for assembly {assembly} found: {options}')
    return rows[0][0]


def build_variant_warehouse_database_name(taxonomy_code, assembly_code):
    if taxonomy_code and assembly_code:
        return f'eva_{taxonomy_code}_{assembly_code}'
    return None


def resolve_existing_variant_warehouse_db_name(metadata_connection_handle, assembly, taxonomy):
    """
    Retrieve an existing database name by combining the taxonomy_code and assembly code registered in the metadata
    database.
    """
    return build_variant_warehouse_database_name(
        get_taxonomy_code_from_metadata(metadata_connection_handle, taxonomy),
        get_assembly_code_from_metadata(metadata_connection_handle, assembly)
    )


# For backward compatibility
get_variant_warehouse_db_name_from_assembly_and_taxonomy = resolve_existing_variant_warehouse_db_name


def get_assembly_code(metadata_connection_handle, assembly):
    assembly_code = get_assembly_code_from_metadata(metadata_connection_handle, assembly)
    if not assembly_code:
        assembly_name = get_ncbi_assembly_name_from_term(assembly)
        # If the assembly is a patch assembly ex: GRCh37.p8, drop the trailing patch i.e., just return grch37
        if is_patch_assembly(assembly):
            assembly_name = re.sub('\\.p[0-9]+$', '', assembly_name.lower())
        assembly_code = re.sub('[^0-9a-zA-Z]+', '', assembly_name.lower())
    return assembly_code


def build_taxonomy_code(scientific_name):
    """Given a scientific name like "Zea mays", the corresponding taxonomy code should be zmays"""
    return scientific_name[0].lower() + re.sub('[^0-9a-zA-Z]+', '', ''.join(scientific_name.split()[1:])).lower()


def get_taxonomy_code(metadata_connection_handle, taxonomy):
    taxonomy_code = get_taxonomy_code_from_metadata(metadata_connection_handle, taxonomy)
    if not taxonomy_code:
        scientific_name = get_scientific_name_from_ensembl(taxonomy)
        taxonomy_code = build_taxonomy_code(scientific_name)
    return taxonomy_code


def resolve_variant_warehouse_db_name(metadata_connection_handle, assembly, taxonomy):
    """
    Retrieve the database name for this taxonomy/assembly pair whether it exists or not.
    It will use existing taxonomy code or assembly code if available in the metadata database.
    """
    taxonomy_code = get_taxonomy_code(metadata_connection_handle, taxonomy)
    assembly_code = get_assembly_code(metadata_connection_handle, assembly)
    return build_variant_warehouse_database_name(taxonomy_code, assembly_code)


def insert_new_assembly_and_taxonomy(metadata_connection_handle, assembly_accession, taxonomy_id, eva_species_name=None,
                                     in_accessioning=True):
    """
    This script adds new assemblies and taxonomies to EVAPRO.
    You can also add the assembly with a different taxonomy if you provide the
    taxonomy parameters. Example taxonomy page:
    https://www.ebi.ac.uk/ena/data/view/Taxon:9031

    :param assembly_accession: Assembly accession (Example: GCA_000002315.3)
    :param metadata_connection_handle: Metadata DB connection
    :param taxonomy_id: Taxonomy id (Example: 9031)
    :param eva_species_name: EVA species name (Example: chicken).
        Not required if the taxonomy exists or ENA has a common name available.
    :param in_accessioning: Flag that this assembly is in the accessioning data store.
    """
    assembly_name = get_ncbi_assembly_name_from_term(assembly_accession)

    # check if assembly is already in EVAPRO, adding it if not
    assembly_set_id = get_assembly_set_from_metadata(metadata_connection_handle, taxonomy_id, assembly_accession)
    if assembly_set_id is None:
        ensure_taxonomy_is_in_evapro(metadata_connection_handle, taxonomy_id, eva_species_name)
        assembly_code = get_assembly_code(metadata_connection_handle, assembly_accession)
        insert_assembly_in_evapro(metadata_connection_handle, taxonomy_id, assembly_accession, assembly_name, assembly_code)

    update_accessioning_status(metadata_connection_handle, assembly_accession, in_accessioning)
    metadata_connection_handle.commit()


def ensure_taxonomy_is_in_evapro(metadata_connection_handle, taxonomy, eva_species_name=None):
    if is_taxonomy_in_evapro(metadata_connection_handle, taxonomy):
        logger.debug('Taxonomy {} is already in the database'.format(taxonomy))
    else:
        logger.info("Taxonomy {} not present in EVAPRO. Adding taxonomy ...".format(taxonomy))
        scientific_name, common_name = get_scientific_name_and_common_name(taxonomy)
        taxonomy_code = build_taxonomy_code(scientific_name)
        # If a common name cannot be found then we should  use the scientific name
        eva_species_name = eva_species_name or common_name or scientific_name
        insert_taxonomy(metadata_connection_handle, taxonomy, scientific_name, common_name, taxonomy_code, eva_species_name)


def insert_assembly_in_evapro(metadata_connection_handle, taxonomy_id, assembly_accession, assembly_name, assembly_code):
    cur = metadata_connection_handle.cursor()
    cur.execute('INSERT INTO evapro.assembly_set(taxonomy_id, assembly_name, assembly_code) VALUES (%s, %s, %s)',
                (taxonomy_id, assembly_name, assembly_code))

    # get the assembly_set_id that was autogenerated in the row that we just inserted in assembly_set
    assembly_set_id = get_all_results_for_query(metadata_connection_handle,
                                    'SELECT assembly_set_id FROM evapro.assembly_set '
                                    'WHERE taxonomy_id={} and assembly_name=\'{}\' and assembly_code=\'{}\''
                                                .format(taxonomy_id, assembly_name, assembly_code))[0][0]

    assembly_chain = assembly_accession.split('.')[0]
    assembly_version = assembly_accession.split('.')[1]
    cur.execute('INSERT INTO evapro.accessioned_assembly('
                'assembly_set_id, assembly_accession, assembly_chain, assembly_version) VALUES (%s,%s,%s,%s)',
                (assembly_set_id, assembly_accession, assembly_chain, assembly_version))

    logger.info('New assembly added with assembly_set_id: {0}'.format(assembly_set_id))
    return assembly_set_id


def update_accessioning_status(metadata_connection_handle, assembly_accession, in_accessioning_flag):
    cur = metadata_connection_handle.cursor()
    # Only insert assembly accessions which are NOT already in the assembly_accessioning_store_status table
    assembly_accessioning_store_insert_query = "INSERT INTO evapro.assembly_accessioning_store_status " \
                                               "SELECT * FROM (SELECT " \
                                               "cast('{0}' as text) as assembly_accession" \
                                               ", cast('{1}' as boolean) as loaded) temp " \
                                               "WHERE assembly_accession NOT IN " \
                                               "(SELECT assembly_accession FROM " \
                                               "evapro.assembly_accessioning_store_status)" \
                                               .format(assembly_accession, in_accessioning_flag)
    cur.execute(assembly_accessioning_store_insert_query)


def get_assembly_set_from_metadata(metadata_connection_handle, taxonomy, assembly_accession):
    query = (f"SELECT acc.assembly_set_id "
             f"FROM evapro.accessioned_assembly acc "
             f"JOIN assembly_set asm on acc.assembly_set_id = asm.assembly_set_id "
             f"WHERE assembly_accession='{assembly_accession}' AND taxonomy_id={taxonomy}")
    rows = get_all_results_for_query(metadata_connection_handle, query)

    if len(rows) == 1:
        return rows[0][0]
    elif len(rows) == 0:
        return None
    else:
        raise ValueError('Inconsistent database state: several assembly_set_ids for the same taxonomy ({}) and '
                         'assembly accession ({}): {}'.format(taxonomy, assembly_accession, rows))


def is_taxonomy_in_evapro(metadata_connection_handle, taxonomy_id):
    taxonomy_query = 'SELECT taxonomy_id FROM evapro.taxonomy WHERE taxonomy_id={}'.format(taxonomy_id)
    taxonomy_ids_in_evapro = get_all_results_for_query(metadata_connection_handle, taxonomy_query)
    return len(taxonomy_ids_in_evapro) > 0


def insert_taxonomy(metadata_connection_handle, taxonomy_id, scientific_name, common_name, taxonomy_code, eva_species_name):
    if taxonomy_code is None or eva_species_name is None:
        raise ValueError('Error: taxonomy code ({}) and EVA taxonomy name ({}) are required '
                         'for inserting a taxonomy'.format(taxonomy_code, eva_species_name))
    cur = metadata_connection_handle.cursor()
    cur.execute('INSERT INTO evapro.taxonomy(taxonomy_id, common_name, scientific_name, taxonomy_code, eva_name) '
                'VALUES (%s, %s, %s, %s, %s)',
                (taxonomy_id, common_name, scientific_name, taxonomy_code, eva_species_name))
    logger.info('New taxonomy {} added'.format(taxonomy_id))


def add_to_supported_assemblies(metadata_connection_handle, source_of_assembly: str, target_assembly: str,
                                taxonomy_id: int):
    today = datetime.date.today().strftime('%Y-%m-%d')
    # First check if the current assembly is already target - if so don't do anything
    current_query = (
        f"SELECT assembly_id FROM {SUPPORTED_ASSEMBLY_TRACKER_TABLE} "
        f"WHERE taxonomy_id={taxonomy_id} AND current=true;"
    )
    results = get_all_results_for_query(metadata_connection_handle, current_query)
    if len(results) > 0 and results[0][0] == target_assembly:
        logger.warning(f'Current assembly for taxonomy {taxonomy_id} is already {target_assembly}!')
        return

    # Deprecate the last current assembly
    update_query = (
        f"UPDATE {SUPPORTED_ASSEMBLY_TRACKER_TABLE} "
        f"SET current=false, end_date='{today}' "
        f"WHERE taxonomy_id={taxonomy_id} AND current=true;"
    )
    execute_query(metadata_connection_handle, update_query)

    # Then insert the new assembly
    insert_query = (
        f"INSERT INTO {SUPPORTED_ASSEMBLY_TRACKER_TABLE} "
        f"(taxonomy_id, source, assembly_id, current, start_date) "
        f"VALUES({taxonomy_id}, '{source_of_assembly}', '{target_assembly}', true, '{today}');"
    )
    execute_query(metadata_connection_handle, insert_query)


# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
import subprocess

from ebi_eva_common_pyutils.logger import logging_config as log_cfg

logger = log_cfg.get_logger(__name__)


def run_command_with_output(command_description, command, return_process_output=False,
                            log_error_stream_to_output=False, stdout_log_level=logging.INFO,
                            stderr_log_level=logging.ERROR):
    process_output = ""

    logger.log(stdout_log_level, "Starting process: " + command_description)
    logger.log(stdout_log_level, "Running command: " + command)

    stdout = subprocess.PIPE
    # Some lame utilities like mongodump and mongorestore output non-error messages to error stream
    # This is a workaround for that
    stderr = subprocess.STDOUT if log_error_stream_to_output else subprocess.PIPE

    with subprocess.Popen(command, stdout=stdout, stderr=stderr, bufsize=1, universal_newlines=True,
                          shell=True) as process:
        for line in iter(process.stdout.readline, ''):
            line = str(line).rstrip()
            logger.log(stdout_log_level, line)
            if return_process_output:
                process_output += line + "\n"
        if not log_error_stream_to_output:
            for line in iter(process.stderr.readline, ''):
                line = str(line).rstrip()
                logger.log(stderr_log_level, line)
    if process.returncode != 0:
        logger.error(command_description + " failed! Refer to the error messages for details.")
        raise subprocess.CalledProcessError(process.returncode, process.args)
    else:
        logger.log(stdout_log_level, command_description + " - completed successfully")
    if return_process_output:
        return process_output





# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from enum import Enum
from ebi_eva_common_pyutils.command_utils import run_command_with_output


class FileDiffOption(Enum):
    NOT_IN = 1
    COMMON = 2


def file_diff(file1_path: str, file2_path: str, diff_option: FileDiffOption, output_file_path: str):
    if diff_option == FileDiffOption.NOT_IN:
        run_command_with_output("Finding entries in {0} not in {1}".format(file1_path, file2_path),
                                "comm -23 {0} {1} > {2}".format(file1_path, file2_path, output_file_path))
    elif diff_option == FileDiffOption.COMMON:
        run_command_with_output("Finding entries common to {0} and {1}".format(file1_path, file2_path),
                                "comm -12 {0} {1} > {2}".format(file1_path, file2_path, output_file_path))
